<?php

namespace Drupal\node_revisions_autoclean\Services;

use Drupal\Core\Entity\EntityTypeManager;
use Drupal\Core\Queue\QueueFactory;
use Drupal\node\Entity\Node;

/**
 * Class RevisionsManager.
 */
class RevisionsManager {
  /**
   * Drupal\Core\Entity\EntityTypeManager definition.
   *
   * @var \Drupal\Core\Entity\EntityTypeManager
   */
  protected $entityTypeManager;

  /**
   * Drupal\Core\Queue\QueueFactory definition.
   *
   * @var Drupal\Core\Queue\QueueFactory
   */
  protected $queueFactory;

  /**
   * Constructs a new RevisionsManager object.
   */
  public function __construct(EntityTypeManager $entity_type_manager, QueueFactory $queueFactory) {
    $this->entityTypeManager = $entity_type_manager;
    $this->queueFactory = $queueFactory;
  }

  /**
   * Loads revisions of a node.
   *
   * @param Drupal\node\Entity\Node $entity
   *   Node to load revisions.
   *
   * @return Drupal\node\Entity\Node[]
   *   Returns all revisions.
   */
  public function loadRevisions(Node $entity) {
    $vids = $this->entityTypeManager->getStorage('node')->revisionIds($entity);
    $revisions = [];
    foreach ($vids as $vid) {
      $revisions[] = $this->entityTypeManager->getStorage('node')->loadRevision($vid);
    }
    return $revisions;
  }

  /**
   * Loads revisions to delete of a node according to settings.
   *
   * @param Drupal\node\Entity\Node $node
   *   The node.
   * @param Drupal\node\Entity\Node[] $revisions
   *   Revisions if they are already loaded.
   *
   * @return array
   *   Returns all revisions IDs to delete.
   */
  public function revisionsToDelete(Node $node, array $revisions = []) {
    $settings = \Drupal::config('node_revisions_autoclean.settings');
    $max = $settings->get('node.' . $node->bundle());
    $szDi = $settings->get('interval.' . $node->bundle());
    $minDate = FALSE;
    try {
      $di = new \DateInterval($szDi);
      $minDate = new \DateTime();
      $minDate->sub($di);
    }
    catch (\Exception $exc) {

    }

    $result = [];
    if ($max) {
      if (!count($revisions)) {
        $revisions = $this->loadRevisions($node);
      }
      $revisions = array_reverse($revisions);
      $count = 0;
      $bStart = FALSE;
      /* @var $revision Node */
      foreach ($revisions as $revision) {
        if ($bStart) {
          $count++;
        }
        if ($revision->get('status')->value) {
          $bStart = TRUE;
        }
        if ($minDate && $minDate instanceof \DateTime && $count > $max) {
          if ($revision->getRevisionCreationTime() < $minDate->getTimestamp()) {
            $result[] = $revision->vid->value;
          }
        }
        elseif ($count > $max) {
          $result[] = $revision->vid->value;
        }
      }

    }
    return $result;
  }

  /**
   * Deletes a revision.
   *
   * @param int $revisionID
   *   Revision ID to delete.
   */
  public function deleteRevision($revisionID) {
    $this->deleteRevisions([$revisionID]);
  }

  /**
   * Deletes revisions.
   *
   * @param int[] $revisionsIDs
   *   Revisions IDs to delete.
   */
  public function deleteRevisions(array $revisionsIDs) {
    foreach ($revisionsIDs as $revisionID) {
      $this->entityTypeManager->getStorage('node')->deleteRevision($revisionID);
    }
  }

  /**
   * Queues a node for cronjob.
   *
   * @param Drupal\node\Entity\Node $node
   *   The node.
   */
  public function queueNodeForCronJob(Node $node) {
    $queue = $this->queueFactory->get('cleanup_revisions_worker');
    $queue->createItem((object) [
      'node' => $node,
    ]);
  }

}
